@@ -63,25 +63,6 @@ else:
     import termios
     import select
 
-class PythonExtension(lt.torrent_plugin):
-    def __init__(self, alerts):
-        lt.torrent_plugin.__init__(self)
-        self.alerts = alerts
-        self.alerts.append('PythonExtension')
-        self.count = 0
-
-    def on_piece_pass(self, index):
-        self.alerts.append('got piece %d' % index)
-
-    def on_piece_failed(self, index):
-        self.alerts.append('failed piece %d' % index)
-
-    def tick(self):
-        self.count += 1
-        if self.count >= 10:
-            self.count = 0
-            self.alerts.append('PythonExtension tick')
-
 def write_line(console, line):
     console.write(line)
 
@@ -178,9 +159,6 @@ def main():
     parser.add_option('-p', '--port', 
         type='int', help='set listening port')
 
-    parser.add_option('-r', '--ratio', 
-        type='float', help='set the preferred upload/download ratio. 0 means infinite. Values smaller than 1 are clamped to 1')
-
     parser.add_option('-d', '--max-download-rate', 
         type='float', help='the maximum download rate given in kB/s. 0 means infinite.')
 
@@ -193,13 +171,16 @@ def main():
     parser.add_option('-a', '--allocation-mode', 
         type='string', help='sets mode used for allocating the downloaded files on disk. Possible options are [full | compact]')
 
+    parser.add_option('-r', '--proxy-host', 
+        type='string', help='sets HTTP proxy host and port (separated by \':\')')
+
     parser.set_defaults(
         port=6881
-      , ratio=0
       , max_download_rate=0
       , max_upload_rate=0
       , save_path='./'
       , allocation_mode='compact'
+      , proxy_host=''
     )
 
     (options, args) = parser.parse_args()
@@ -225,35 +206,39 @@ def main():
     ses.set_upload_rate_limit(int(options.max_upload_rate))
     ses.listen_on(options.port, options.port + 10)
     ses.set_settings(settings)
-#   ses.set_severity_level(lt.alert.severity_levels.info)
     ses.set_alert_mask(0xfffffff)
-#    ses.add_extension(lt.create_ut_pex_plugin)
-#    ses.add_extension(lt.create_ut_metadata_plugin)
-#    ses.add_extension(lt.create_metadata_plugin)
+
+    if options.proxy_host != '':
+        ps = lt.proxy_settings()
+        ps.type = lt.proxy_type.http
+        ps.hostname = options.proxy_host.split(':')[0]
+        ps.port = int(options.proxy_host.split(':')[1])
+        ses.set_proxy(ps)
 
     handles = []
     alerts = []
 
-    # Extensions
-    # ses.add_extension(lambda x: PythonExtension(alerts))
-
     for f in args:
-        e = lt.bdecode(open(f, 'rb').read())
-        info = lt.torrent_info(e)
-        print('Adding \'%s\'...' % info.name())
 
         atp = {}
-        try:
-            atp["resume_data"] = open(os.path.join(options.save_path, info.name() + '.fastresume'), 'rb').read()
-        except:
-            pass
-
-        atp["ti"] = info
         atp["save_path"] = options.save_path
         atp["storage_mode"] = lt.storage_mode_t.storage_mode_sparse
         atp["paused"] = False
         atp["auto_managed"] = True
         atp["duplicate_is_error"] = True
+        if f.startswith('magnet:') or f.startswith('http://') or f.startswith('https://'):
+            atp["url"] = f
+        else:
+            e = lt.bdecode(open(f, 'rb').read())
+            info = lt.torrent_info(e)
+            print('Adding \'%s\'...' % info.name())
+
+            try:
+                atp["resume_data"] = open(os.path.join(options.save_path, info.name() + '.fastresume'), 'rb').read()
+            except:
+                pass
+
+            atp["ti"] = info
 
         h = ses.add_torrent(atp)
 
@@ -261,7 +246,6 @@ def main():
 
         h.set_max_connections(60)
         h.set_max_uploads(-1)
-        h.set_ratio(options.ratio)
 
     if os.name == 'nt':
         console = WindowsConsole()
@@ -302,7 +286,6 @@ def main():
                 % (add_suffix(s.download_rate), add_suffix(s.total_download))
             out += 'upload: %s/s (%s) ' \
                 % (add_suffix(s.upload_rate), add_suffix(s.total_upload))
-            out += 'ratio: %s\n' % '0'
 
             if s.state != lt.torrent_status.seeding:
                 out += 'info-hash: %s\n' % h.info_hash()
@@ -314,14 +297,18 @@ def main():
             print_peer_info(console, h.get_peer_info())
             print_download_queue(console, h.get_download_queue())
 
-            if True and s.state != lt.torrent_status.seeding:
-                out = '\n'
-                fp = h.file_progress()
-                ti = h.get_torrent_info()
-                for f,p in zip(ti.files(), fp):
-                    out += progress_bar(p / f.size, 20)
-                    out += ' ' + f.path + '\n'
-                write_line(console, out)
+            if s.state != lt.torrent_status.seeding:
+                try:
+                    out = '\n'
+                    fp = h.file_progress()
+                    fp = 0
+                    ti = h.get_torrent_info()
+                    for f,p in zip(ti.files(), fp):
+                        out += progress_bar(p / f.size, 20)
+                        out += ' ' + f.path + '\n'
+                    write_line(console, out)
+                except:
+                    pass
 
         write_line(console, 76 * '-' + '\n')
         write_line(console, '(q)uit), (p)ause), (u)npause), (r)eannounce\n')
