@@ -6,6 +6,7 @@
 #define BOOST_PYTHON_USE_GCC_SYMBOL_VISIBILITY 1
 #endif
 
+#include "libtorrent/config.hpp"
 #include <boost/python/module.hpp>
 
 void bind_utility();
@@ -14,14 +15,13 @@ void bind_big_number();
 void bind_session();
 void bind_entry();
 void bind_torrent_info();
+void bind_unicode_string_conversion();
 void bind_torrent_handle();
 void bind_torrent_status();
 void bind_session_settings();
 void bind_version();
 void bind_alert();
 void bind_datetime();
-void bind_extensions();
-void bind_peer_plugin();
 void bind_torrent();
 void bind_peer_info();
 void bind_ip_filter();
@@ -42,16 +42,15 @@ BOOST_PYTHON_MODULE(libtorrent)
     bind_entry();
     bind_session();
     bind_torrent_info();
+#if TORRENT_USE_WSTRING
+    bind_unicode_string_conversion();
+#endif
     bind_torrent_handle();
     bind_torrent_status();
     bind_session_settings();
     bind_version();
     bind_alert();
     bind_datetime();
-    bind_extensions();
-#ifndef TORRENT_NO_PYTHON_PLUGINS
-    bind_peer_plugin();
-#endif
     bind_torrent();
     bind_peer_info();
     bind_ip_filter();
