@@ -11,6 +11,14 @@
 #include <libtorrent/storage.hpp>
 #include <libtorrent/ip_filter.hpp>
 #include <libtorrent/disk_io_thread.hpp>
+#include <libtorrent/extensions.hpp>
+
+#include <libtorrent/extensions/lt_trackers.hpp>
+#include <libtorrent/extensions/metadata_transfer.hpp>
+#include <libtorrent/extensions/smart_ban.hpp>
+#include <libtorrent/extensions/ut_metadata.hpp>
+#include <libtorrent/extensions/ut_pex.hpp>
+
 #include "gil.hpp"
 
 using namespace boost::python;
@@ -21,9 +29,9 @@ namespace
     void listen_on(session& s, int min_, int max_, char const* interface, int flags)
     {
         allow_threading_guard guard;
-		  error_code ec;
+        error_code ec;
         s.listen_on(std::make_pair(min_, max_), ec, interface, flags);
-		  if (ec) throw libtorrent_exception(ec);
+        if (ec) throw libtorrent_exception(ec);
     }
 
     void outgoing_ports(session& s, int _min, int _max)
@@ -42,27 +50,31 @@ namespace
     }
 #endif
 
-    struct invoke_extension_factory
-    {
-        invoke_extension_factory(object const& callback)
-            : cb(callback)
-        {}
-
-        boost::shared_ptr<torrent_plugin> operator()(torrent* t, void*)
-        {
-           lock_gil lock;
-           return extract<boost::shared_ptr<torrent_plugin> >(cb(ptr(t)))();
-        }
-
-        object cb;
-    };
-
     void add_extension(session& s, object const& e)
     {
-        allow_threading_guard guard;
-        s.add_extension(invoke_extension_factory(e));
+       if (!extract<std::string>(e).check()) return;
+
+       std::string name = extract<std::string>(e);
+       if (name == "ut_metadata")
+            s.add_extension(create_ut_metadata_plugin);
+       else if (name == "ut_pex")
+            s.add_extension(create_ut_pex_plugin);
+       else if (name == "smart_ban")
+            s.add_extension(create_smart_ban_plugin);
+       else if (name == "lt_trackers")
+            s.add_extension(create_lt_trackers_plugin);
+       else if (name == "metadata_transfer")
+            s.add_extension(create_metadata_plugin);
     }
 
+#ifndef TORRENT_NO_DEPRECATE
+
+    boost::shared_ptr<torrent_plugin> dummy_plugin_wrapper(torrent* t) {
+        return boost::shared_ptr<torrent_plugin>();
+    }
+
+#endif
+
 	void session_set_settings(session& ses, dict const& sett_dict)
 	{
 		bencode_map_entry* map;
@@ -96,14 +108,21 @@ namespace
 			}
 		}
 
+		if (!sett_dict.has_key("outgoing_port"))
+			sett.outgoing_ports.first = extract<int>(sett_dict["outgoing_port"]);
+		if (!sett_dict.has_key("num_outgoing_ports"))
+			sett.outgoing_ports.second = sett.outgoing_ports.first + extract<int>(sett_dict["num_outgoing_ports"]);
+
 		ses.set_settings(sett);
 	}
 
 	dict session_get_settings(session const& ses)
 	{
-		allow_threading_guard guard;
-		 
-		session_settings sett = ses.settings();
+		session_settings sett;
+		{
+			allow_threading_guard guard;
+			sett = ses.settings();
+		}
 		dict sett_dict;
 		bencode_map_entry* map;
 		int len;
@@ -131,6 +150,8 @@ namespace
 					break;
 			}
 		}
+		sett_dict["outgoing_port"] = sett.outgoing_ports.first;
+		sett_dict["num_outgoing_ports"] = sett.outgoing_ports.second - sett.outgoing_ports.first + 1;
 		return sett_dict;
 	}
 
@@ -148,7 +169,7 @@ namespace
 }
 
     void dict_to_add_torrent_params(dict params, add_torrent_params& p
-        , std::vector<char>& rd, std::list<std::string>& string_storage)
+        , std::vector<char>& rd)
     {
         // torrent_info objects are always held by an intrusive_ptr in the python binding
         if (params.has_key("ti") && params.get("ti") != boost::python::object())
@@ -157,10 +178,7 @@ namespace
         if (params.has_key("info_hash"))
             p.info_hash = extract<sha1_hash>(params["info_hash"]);
         if (params.has_key("name"))
-        {
-            string_storage.push_back(extract<std::string>(params["name"]));
-            p.name = string_storage.back().c_str();
-        }
+            p.name = extract<std::string>(params["name"]);
         p.save_path = extract<std::string>(params["save_path"]);
 
         if (params.has_key("resume_data"))
@@ -173,7 +191,7 @@ namespace
         if (params.has_key("storage_mode"))
             p.storage_mode = extract<storage_mode_t>(params["storage_mode"]);
 
-        if (params.has_key("tracker_url"))
+        if (params.has_key("trackers"))
         {
             list l = extract<list>(params["trackers"]);
             int n = boost::python::len(l);
@@ -191,10 +209,7 @@ namespace
 #ifndef TORRENT_NO_DEPRECATE
         std::string url;
         if (params.has_key("tracker_url"))
-        {
-            string_storage.push_back(extract<std::string>(params["tracker_url"]));
-            p.tracker_url = string_storage.back().c_str();
-        }
+            p.trackers.push_back(extract<std::string>(params["tracker_url"]));
         if (params.has_key("seed_mode"))
             p.seed_mode = params["seed_mode"];
         if (params.has_key("upload_mode"))
@@ -234,8 +249,7 @@ namespace
     {
         add_torrent_params p;
         std::vector<char> resume_buf;
-        std::list<std::string> string_buf;
-        dict_to_add_torrent_params(params, p, resume_buf, string_buf);
+        dict_to_add_torrent_params(params, p, resume_buf);
 
         allow_threading_guard guard;
 
@@ -247,9 +261,24 @@ namespace
 #endif
     }
 
+    void async_add_torrent(session& s, dict params)
+    {
+        add_torrent_params p;
+        std::vector<char> resume_buf;
+        dict_to_add_torrent_params(params, p, resume_buf);
+
+        allow_threading_guard guard;
+
+#ifndef BOOST_NO_EXCEPTIONS
+        s.async_add_torrent(p);
+#else
+        error_code ec;
+        s.async_add_torrent(p, ec);
+#endif
+    }
+
     void dict_to_feed_settings(dict params, feed_settings& feed
-        , std::vector<char>& resume_buf
-        , std::list<std::string>& string_storage)
+        , std::vector<char>& resume_buf)
     {
         if (params.has_key("auto_download"))
             feed.auto_download = extract<bool>(params["auto_download"]);
@@ -259,28 +288,28 @@ namespace
             feed.url = extract<std::string>(params["url"]);
         if (params.has_key("add_args"))
             dict_to_add_torrent_params(dict(params["add_args"]), feed.add_args
-                , resume_buf, string_storage);
+                , resume_buf);
     }
 
     feed_handle add_feed(session& s, dict params)
     {
-        allow_threading_guard guard;
-
         feed_settings feed;
         // this static here is a bit of a hack. It will
         // probably work for the most part
         static std::vector<char> resume_buf;
-        std::list<std::string> string_storage;
-        dict_to_feed_settings(params, feed, resume_buf, string_storage);
+        dict_to_feed_settings(params, feed, resume_buf);
 
+        allow_threading_guard guard;
         return s.add_feed(feed);
     }
 
     dict get_feed_status(feed_handle const& h)
     {
-        allow_threading_guard guard;
-
-        feed_status s = h.get_feed_status();
+        feed_status s;
+        {
+            allow_threading_guard guard;
+            s = h.get_feed_status();
+        }
         dict ret;
         ret["url"] = s.url;
         ret["title"] = s.title;
@@ -307,33 +336,32 @@ namespace
             item["info_hash"] = i->info_hash.to_string();
             items.append(item);
         }
-		  ret["items"] = items;
+        ret["items"] = items;
         return ret;
     }
 
     void set_feed_settings(feed_handle& h, dict sett)
     {
-        allow_threading_guard guard;
-
         feed_settings feed;
         static std::vector<char> resume_buf;
-        std::list<std::string> string_storage;
-        dict_to_feed_settings(sett, feed, resume_buf, string_storage);
+        dict_to_feed_settings(sett, feed, resume_buf);
         h.set_settings(feed);
     }
 
     dict get_feed_settings(feed_handle& h)
     {
-        allow_threading_guard guard;
-
-        feed_settings s = h.settings();
+        feed_settings s;
+        {
+            allow_threading_guard guard;
+            s = h.settings();
+        }
         dict ret;
         ret["url"] = s.url;
         ret["auto_download"] = s.auto_download;
         ret["default_ttl"] = s.default_ttl;
         return ret;
     }
-	
+
     void start_natpmp(session& s)
     {
         allow_threading_guard guard;
@@ -354,9 +382,12 @@ namespace
 
     list get_torrents(session& s)
     {
-        allow_threading_guard guard;
         list ret;
-        std::vector<torrent_handle> torrents = s.get_torrents();
+        std::vector<torrent_handle> torrents;
+        {
+           allow_threading_guard guard;
+           torrents = s.get_torrents();
+        }
 
         for (std::vector<torrent_handle>::iterator i = torrents.begin(); i != torrents.end(); ++i)
         {
@@ -387,6 +418,24 @@ namespace
         return e;
     }
 
+    list pop_alerts(session& ses)
+    {
+        std::deque<alert*> alerts;
+        {
+            allow_threading_guard guard;
+            ses.pop_alerts(&alerts);
+        }
+
+        list ret;
+        for (std::deque<alert*>::iterator i = alerts.begin()
+            , end(alerts.end()); i != end; ++i)
+        {
+				std::auto_ptr<alert> ptr(*i);
+            ret.append(ptr);
+        }
+        return ret;
+    }
+
 } // namespace unnamed
 
 
@@ -495,6 +544,7 @@ void bind_session()
         .value("flag_auto_managed", add_torrent_params::flag_auto_managed)
         .value("flag_duplicate_is_error", add_torrent_params::flag_duplicate_is_error)
         .value("flag_merge_resume_trackers", add_torrent_params::flag_merge_resume_trackers)
+        .value("flag_update_subscribe", add_torrent_params::flag_update_subscribe)
     ;
     class_<cache_status>("cache_status")
         .def_readonly("blocks_written", &cache_status::blocks_written)
@@ -538,6 +588,7 @@ void bind_session()
 #endif
 #endif
         .def("add_torrent", &add_torrent)
+        .def("async_add_torrent", &async_add_torrent)
 #ifndef BOOST_NO_EXCEPTIONS
 #ifndef TORRENT_NO_DEPRECATE
         .def(
@@ -575,6 +626,7 @@ void bind_session()
         .def("get_settings", &session_get_settings)
 #else
         .def("settings", &session_get_settings)
+        .def("get_settings", &session_get_settings)
 #endif
         .def("set_settings", &session_set_settings)
 #ifndef TORRENT_DISABLE_ENCRYPTION
@@ -594,6 +646,7 @@ void bind_session()
 #endif
         .def("set_alert_mask", allow_threads(&session::set_alert_mask))
         .def("pop_alert", allow_threads(&session::pop_alert))
+        .def("pop_alerts", &pop_alerts)
         .def("wait_for_alert", &wait_for_alert, return_internal_reference<>())
         .def("add_extension", &add_extension)
 #ifndef TORRENT_NO_DEPRECATE
@@ -641,7 +694,9 @@ void bind_session()
     ;
 
     enum_<session::listen_on_flags_t>("listen_on_flags_t")
+#ifndef TORRENT_NO_DEPRECATE
         .value("listen_reuse_address", session::listen_reuse_address)
+#endif
         .value("listen_no_system_port", session::listen_no_system_port)
     ;
 
@@ -652,8 +707,20 @@ void bind_session()
         .def("settings", &get_feed_settings)
     ;
 
+#ifndef TORRENT_NO_DEPRECATE
+    def("create_ut_pex_plugin", dummy_plugin_wrapper);
+    def("create_metadata_plugin", dummy_plugin_wrapper);
+    def("create_ut_metadata_plugin", dummy_plugin_wrapper);
+    def("create_smart_ban_plugin", dummy_plugin_wrapper);
+#endif
+
     register_ptr_to_python<std::auto_ptr<alert> >();
 
 	 def("high_performance_seed", high_performance_seed);
 	 def("min_memory_usage", min_memory_usage);
+
+	 scope().attr("create_metadata_plugin") = "metadata_transfer";
+	 scope().attr("create_ut_metadata_plugin") = "ut_metadata";
+	 scope().attr("create_ut_pex_plugin") = "ut_pex";
+	 scope().attr("create_smart_ban_plugin") = "smart_ban";
 }
