@@ -42,7 +42,9 @@ void bind_session_settings()
         .def_readwrite("num_want", &session_settings::num_want)
         .def_readwrite("initial_picker_threshold", &session_settings::initial_picker_threshold)
         .def_readwrite("allowed_fast_set_size", &session_settings::allowed_fast_set_size)
+        .def_readwrite("suggest_mode", &session_settings::suggest_mode)
         .def_readwrite("max_queued_disk_bytes", &session_settings::max_queued_disk_bytes)
+        .def_readwrite("max_queued_disk_bytes_low_watermark", &session_settings::max_queued_disk_bytes_low_watermark)
         .def_readwrite("handshake_timeout", &session_settings::handshake_timeout)
 #ifndef TORRENT_DISABLE_DHT
         .def_readwrite("use_dht_as_fallback", &session_settings::use_dht_as_fallback)
@@ -56,11 +58,13 @@ void bind_session_settings()
         .def_readwrite("auto_upload_slots_rate_based", &session_settings::auto_upload_slots_rate_based)
 #endif
         .def_readwrite("choking_algorithm", &session_settings::choking_algorithm)
+        .def_readwrite("seed_choking_algorithm", &session_settings::seed_choking_algorithm)
         .def_readwrite("use_parole_mode", &session_settings::use_parole_mode)
         .def_readwrite("cache_size", &session_settings::cache_size)
         .def_readwrite("cache_buffer_chunk_size", &session_settings::cache_buffer_chunk_size)
         .def_readwrite("cache_expiry", &session_settings::cache_expiry)
         .def_readwrite("use_read_cache", &session_settings::use_read_cache)
+        .def_readwrite("explicit_read_cache", &session_settings::explicit_read_cache)
         .def_readwrite("disk_io_write_mode", &session_settings::disk_io_write_mode)
         .def_readwrite("disk_io_read_mode", &session_settings::disk_io_read_mode)
         .def_readwrite("coalesce_reads", &session_settings::coalesce_reads)
@@ -127,10 +131,11 @@ void bind_session_settings()
         .def_readwrite("increase_est_reciprocation_rate", &session_settings::increase_est_reciprocation_rate)
         .def_readwrite("decrease_est_reciprocation_rate", &session_settings::decrease_est_reciprocation_rate)
         .def_readwrite("incoming_starts_queued_torrents", &session_settings::incoming_starts_queued_torrents)
-        .def_readwrite("report_true_downoaded", &session_settings::report_true_downloaded)
+        .def_readwrite("report_true_downloaded", &session_settings::report_true_downloaded)
         .def_readwrite("strict_end_game_mode", &session_settings::strict_end_game_mode)
         .def_readwrite("broadcast_lsd", &session_settings::broadcast_lsd)
         .def_readwrite("ignore_resume_timestamps", &session_settings::ignore_resume_timestamps)
+        .def_readwrite("no_recheck_incomplete_resume", &session_settings::no_recheck_incomplete_resume)
         .def_readwrite("anonymous_mode", &session_settings::anonymous_mode)
         .def_readwrite("tick_interval", &session_settings::tick_interval)
         .def_readwrite("report_web_seed_downloads", &session_settings::report_web_seed_downloads)
@@ -146,7 +151,7 @@ void bind_session_settings()
         .def_readwrite("connections_limit", &session_settings::connections_limit)
         .def_readwrite("utp_target_delay", &session_settings::utp_target_delay)
         .def_readwrite("utp_gain_factor", &session_settings::utp_gain_factor)
-        .def_readwrite("utp_sun_resends", &session_settings::utp_syn_resends)
+        .def_readwrite("utp_syn_resends", &session_settings::utp_syn_resends)
         .def_readwrite("utp_fin_resends", &session_settings::utp_fin_resends)
         .def_readwrite("utp_num_resends", &session_settings::utp_num_resends)
         .def_readwrite("utp_connect_timeout", &session_settings::utp_connect_timeout)
@@ -162,7 +167,7 @@ void bind_session_settings()
         .def_readwrite("max_metadata_size", &session_settings::max_metadata_size)
         .def_readwrite("smooth_connects", &session_settings::smooth_connects)
         .def_readwrite("always_send_user_agent", &session_settings::always_send_user_agent)
-        .def_readwrite("apply_ip_filter_to_trakers", &session_settings::apply_ip_filter_to_trackers)
+        .def_readwrite("apply_ip_filter_to_trackers", &session_settings::apply_ip_filter_to_trackers)
         .def_readwrite("read_job_every", &session_settings::read_job_every)
         .def_readwrite("use_disk_read_ahead", &session_settings::use_disk_read_ahead)
         .def_readwrite("lock_files", &session_settings::lock_files)
@@ -170,6 +175,7 @@ void bind_session_settings()
         .def_readwrite("enable_incoming_tcp", &session_settings::enable_incoming_tcp)
         .def_readwrite("enable_outgoing_utp", &session_settings::enable_outgoing_utp)
         .def_readwrite("enable_incoming_utp", &session_settings::enable_incoming_utp)
+        .def_readwrite("max_pex_peers", &session_settings::max_pex_peers)
         .def_readwrite("ssl_listen", &session_settings::ssl_listen)
         .def_readwrite("tracker_backoff", &session_settings::tracker_backoff)
     ;
@@ -181,6 +187,7 @@ void bind_session_settings()
         .value("socks5_pw", proxy_settings::socks5_pw)
         .value("http", proxy_settings::http)
         .value("http_pw", proxy_settings::http_pw)
+        .value("i2p_proxy", proxy_settings::i2p_proxy)
     ;
 
     enum_<session_settings::disk_cache_algo_t>("disk_cache_algo_t")
@@ -195,6 +202,17 @@ void bind_session_settings()
         .value("bittyrant_choker", session_settings::bittyrant_choker)
     ;
 
+    enum_<session_settings::seed_choking_algorithm_t>("seed_choking_algorithm_t")
+        .value("round_robin", session_settings::round_robin)
+        .value("fastest_upload", session_settings::fastest_upload)
+        .value("anti_leech", session_settings::anti_leech)
+    ;
+
+    enum_<session_settings::suggest_mode_t>("suggest_mode_t")
+        .value("no_piece_suggestions", session_settings::no_piece_suggestions)
+        .value("suggest_read_cache", session_settings::suggest_read_cache)
+    ;
+
     enum_<session_settings::io_buffer_mode_t>("io_buffer_mode_t")
         .value("enable_os_cache", session_settings::enable_os_cache)
         .value("disable_os_cache_for_aligned_files", session_settings::disable_os_cache_for_aligned_files)
@@ -212,6 +230,8 @@ void bind_session_settings()
         .def_readwrite("password", &proxy_settings::password)
         .def_readwrite("username", &proxy_settings::username)
         .def_readwrite("type", &proxy_settings::type)
+        .def_readwrite("proxy_peer_connections", &proxy_settings::proxy_peer_connections)
+        .def_readwrite("proxy_hostnames", &proxy_settings::proxy_hostnames)
     ;
 
 #ifndef TORRENT_DISABLE_DHT
