@@ -75,7 +75,10 @@ void print_usage()
 		"-w url      adds a web seed to the torrent with\n"
 		"            the specified url\n"
 		"-t url      adds the specified tracker to the\n"
-		"            torrent\n"
+		"            torrent. For multiple trackers, specify more\n"
+		"            -t options\n"
+		"-c comment  sets the comment to the specified string\n"
+		"-C creator  sets the created-by field to the specified string\n"
 		"-p bytes    enables padding files. Files larger\n"
 		"            than bytes will be piece-aligned\n"
 		"-s bytes    specifies a piece size for the torrent\n"
@@ -86,7 +89,7 @@ void print_usage()
 		"            If this is not specified, the torrent file is\n"
 		"            printed to the standard out, except on windows\n"
 		"            where the filename defaults to a.torrent\n"
-		"-c file     add root certificate to the torrent, to verify\n"
+		"-r file     add root certificate to the torrent, to verify\n"
 		"            the HTTPS tracker\n"
 		, stderr);
 }
@@ -95,7 +98,8 @@ int main(int argc, char* argv[])
 {
 	using namespace libtorrent;
 
-	char const* creator_str = "libtorrent";
+	std::string creator_str = "libtorrent";
+	std::string comment_str;
 
 	if (argc < 2)
 	{
@@ -164,7 +168,15 @@ int main(int argc, char* argv[])
 				case 'l':
 					flags |= create_torrent::symlinks;
 					break;
+				case 'C':
+					++i;
+					creator_str = argv[i];
+					break;
 				case 'c':
+					++i;
+					comment_str = argv[i];
+					break;
+				case 'r':
 					++i;
 					root_cert = argv[i];
 					break;
@@ -186,9 +198,10 @@ int main(int argc, char* argv[])
 		}
 
 		create_torrent t(fs, piece_size, pad_file_limit, flags);
+		int tier = 0;
 		for (std::vector<std::string>::iterator i = trackers.begin()
-			, end(trackers.end()); i != end; ++i)
-			t.add_tracker(*i);
+			, end(trackers.end()); i != end; ++i, ++tier)
+			t.add_tracker(*i, tier);
 
 		for (std::vector<std::string>::iterator i = web_seeds.begin()
 			, end(web_seeds.end()); i != end; ++i)
@@ -204,7 +217,9 @@ int main(int argc, char* argv[])
 		}
 
 		fprintf(stderr, "\n");
-		t.set_creator(creator_str);
+		t.set_creator(creator_str.c_str());
+		if (!comment_str.empty())
+			t.set_comment(comment_str.c_str());
 
 		if (!root_cert.empty())
 		{
@@ -226,6 +241,12 @@ int main(int argc, char* argv[])
 		FILE* output = stdout;
 		if (!outfile.empty())
 			output = fopen(outfile.c_str(), "wb+");
+		if (output == NULL)
+		{
+			fprintf(stderr, "failed to open file \"%s\": (%d) %s\n"
+				, outfile.c_str(), errno, strerror(errno));
+			return 1;
+		}
 		fwrite(&torrent[0], 1, torrent.size(), output);
 
 		if (output != stdout)
