@@ -37,6 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <vector>
 #include <set>
 #include <list>
+#include <stdarg.h> // for va_start, va_end
 
 #ifndef TORRENT_DISABLE_GEO_IP
 #ifdef WITH_SHIPPED_GEOIP_H
@@ -733,7 +734,7 @@ namespace libtorrent
 			boost::shared_ptr<socket_type> m_i2p_listen_socket;
 #endif
 
-			void setup_listener(listen_socket_t* s, tcp::endpoint ep, int retries
+			void setup_listener(listen_socket_t* s, tcp::endpoint ep, int& retries
 				, bool v6_only, int flags, error_code& ec);
 
 			// the proxy used for bittorrent
@@ -1149,7 +1150,7 @@ namespace libtorrent
 			void tracker_warning(tracker_request const& req
 				, std::string const& str)
 			{
-				debug_log("*** tracker warning: " + str);
+				debug_log("*** tracker warning: %s", str.c_str());
 			}
 
 			void tracker_response(tracker_request const&
@@ -1180,7 +1181,7 @@ namespace libtorrent
 				}
 				snprintf(tmp, 200, "external ip: %s\n", print_address(external_ip).c_str());
 				s += tmp;
-				debug_log(s);
+				debug_log("%s", s.c_str());
 			}
 
 			void tracker_request_timed_out(
@@ -1193,15 +1194,23 @@ namespace libtorrent
 				, int response_code, error_code const& ec, const std::string& str
 				, int retry_interval)
 			{
-				char msg[256];
-				snprintf(msg, sizeof(msg), "*** tracker error: %d: %s %s"
+				debug_log("*** tracker error: %d: %s %s"
 					, response_code, ec.message().c_str(), str.c_str());
-				debug_log(msg);
 			}
 			
-			void debug_log(const std::string& line)
+			void debug_log(const char* fmt, ...) const
 			{
-				(*m_ses.m_logger) << time_now_string() << " " << line << "\n";
+				if (!m_ses.m_logger) return;
+
+				va_list v;	
+				va_start(v, fmt);
+	
+				char usr[1024];
+				vsnprintf(usr, sizeof(usr), fmt, v);
+				va_end(v);
+				char buf[1280];
+				snprintf(buf, sizeof(buf), "%s: %s\n", time_now_string(), usr);
+				(*m_ses.m_logger) << buf;
 			}
 			session_impl& m_ses;
 		};
