@@ -142,9 +142,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #pragma warning(disable:4251)
 // '_vsnprintf': This function or variable may be unsafe
 #pragma warning(disable:4996)
-// 'strdup': The POSIX name for this item is deprecated. Instead, use the ISO C++ conformant name: _strdup
-#pragma warning(disable: 4996)
-#define strdup _strdup
 
 #define TORRENT_DEPRECATED_PREFIX __declspec(deprecated)
 
@@ -366,11 +363,11 @@ inline int snprintf(char* buf, int len, char const* fmt, ...)
 #endif
 
 #ifndef TORRENT_USE_WSTRING
-#if defined UNICODE && !defined BOOST_NO_STD_WSTRING
+#if !defined BOOST_NO_STD_WSTRING
 #define TORRENT_USE_WSTRING 1
 #else
 #define TORRENT_USE_WSTRING 0
-#endif // UNICODE
+#endif // BOOST_NO_STD_WSTRING
 #endif // TORRENT_USE_WSTRING
 
 #ifndef TORRENT_HAS_FALLOCATE
@@ -443,10 +440,6 @@ inline int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_USE_I2P 1
 #endif
 
-#ifndef TORRENT_HAS_STRDUP
-#define TORRENT_HAS_STRDUP 1
-#endif
-
 #if !defined TORRENT_IOV_MAX
 #ifdef IOV_MAX
 #define TORRENT_IOV_MAX IOV_MAX
@@ -500,17 +493,6 @@ inline int snprintf(char* buf, int len, char const* fmt, ...)
 
 #endif
 
-#if !TORRENT_HAS_STRDUP
-inline char* strdup(char const* str)
-{
-	if (str == 0) return 0;
-	char* tmp = (char*)malloc(strlen(str) + 1);
-	if (tmp == 0) return 0;
-	strcpy(tmp, str);
-	return tmp;
-}
-#endif
-
 // for non-exception builds
 #ifdef BOOST_NO_EXCEPTIONS
 #define TORRENT_TRY if (true)
