@@ -40,9 +40,9 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/config.hpp"
 #include "libtorrent/assert.hpp"
-#include "libtorrent/escape_string.hpp"
 
 #if TORRENT_USE_IOSTREAM
+#include "libtorrent/escape_string.hpp" // to_hex, from_hex
 #include <iostream>
 #include <iomanip>
 #endif
@@ -67,6 +67,20 @@ namespace libtorrent
 
 		big_number() { clear(); }
 
+		static big_number max()
+		{
+			big_number ret;
+			memset(ret.m_number, 0xff, size);
+			return ret;
+		}
+
+		static big_number min()
+		{
+			big_number ret;
+			memset(ret.m_number, 0, size);
+			return ret;
+		}
+
 		explicit big_number(char const* s)
 		{
 			if (s == 0) clear();
@@ -77,14 +91,14 @@ namespace libtorrent
 		{
 			TORRENT_ASSERT(s.size() >= 20);
 			int sl = int(s.size()) < size ? int(s.size()) : size;
-			std::memcpy(m_number, &s[0], sl);
+			std::memcpy(m_number, s.c_str(), sl);
 		}
 
 		void assign(std::string const& s)
 		{
 			TORRENT_ASSERT(s.size() >= 20);
 			int sl = int(s.size()) < size ? int(s.size()) : size;
-			std::memcpy(m_number, &s[0], sl);
+			std::memcpy(m_number, s.c_str(), sl);
 		}
 
 		void assign(char const* str) { std::memcpy(m_number, str, size); }
@@ -97,6 +111,59 @@ namespace libtorrent
 			return true;
 		}
 
+		big_number& operator<<=(int n)
+		{
+			TORRENT_ASSERT(n >= 0);
+			int num_bytes = n / 8;
+			if (num_bytes >= number_size)
+			{
+				std::memset(m_number, 0, number_size);
+				return *this;
+			}
+
+			if (num_bytes > 0)
+			{
+				std::memmove(m_number, m_number + num_bytes, number_size - num_bytes);
+				std::memset(m_number + number_size - num_bytes, 0, num_bytes);
+				n -= num_bytes * 8;
+			}
+			if (n > 0)
+			{
+				for (int i = 0; i < number_size - 1; ++i)
+				{
+					m_number[i] <<= n;
+					m_number[i] |= m_number[i+1] >> (8 - n);
+				}
+			}
+			return *this;
+		}
+
+		big_number& operator>>=(int n)
+		{
+			TORRENT_ASSERT(n >= 0);
+			int num_bytes = n / 8;
+			if (num_bytes >= number_size)
+			{
+				std::memset(m_number, 0, number_size);
+				return *this;
+			}
+			if (num_bytes > 0)
+			{
+				std::memmove(m_number + num_bytes, m_number, number_size - num_bytes);
+				std::memset(m_number, 0, num_bytes);
+				n -= num_bytes * 8;
+			}
+			if (n > 0)
+			{
+				for (int i = number_size - 1; i > 0; --i)
+				{
+					m_number[i] >>= n;
+					m_number[i] |= m_number[i-1] << (8 - n);
+				}
+			}
+			return *this;
+		}
+
 		bool operator==(big_number const& n) const
 		{
 			return std::equal(n.m_number, n.m_number+number_size, m_number);
@@ -125,6 +192,20 @@ namespace libtorrent
 			return ret;
 		}
 		
+		big_number operator^ (big_number const& n) const
+		{
+			big_number ret = *this;
+			ret ^= n;
+			return ret;
+		}
+
+		big_number operator& (big_number const& n) const
+		{
+			big_number ret = *this;
+			ret &= n;
+			return ret;
+		}
+
 		big_number& operator &= (big_number const& n)
 		{
 			for (int i = 0; i< number_size; ++i)
