@@ -33,6 +33,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_SOCKS5_STREAM_HPP_INCLUDED
 #define TORRENT_SOCKS5_STREAM_HPP_INCLUDED
 
+#include <boost/function/function1.hpp>
+#include <boost/bind.hpp>
+#include <boost/shared_ptr.hpp>
 #include "libtorrent/proxy_base.hpp"
 
 namespace libtorrent {
@@ -60,7 +63,7 @@ namespace libtorrent {
 typedef asio::error::error_category socks_error_category;
 #else
 
-struct TORRENT_EXPORT socks_error_category : boost::system::error_category
+struct TORRENT_EXTRA_EXPORT socks_error_category : boost::system::error_category
 {
 	virtual const char* name() const;
 	virtual std::string message(int ev) const;
@@ -94,6 +97,29 @@ public:
 		m_password = password;
 	}
 
+	void set_dst_name(std::string const& host)
+	{
+		m_dst_name = host;
+		if (m_dst_name.size() > 255)
+			m_dst_name.resize(255);
+	}
+
+	void close(error_code& ec)
+	{
+		m_hostname.clear();
+		m_dst_name.clear();
+		proxy_base::close(ec);
+	}
+
+#ifndef BOOST_NO_EXCEPTIONS
+	void close()
+	{
+		m_hostname.clear();
+		m_dst_name.clear();
+		proxy_base::close();
+	}
+#endif
+
 	typedef boost::function<void(error_code const&)> handler_type;
 
 //#error fix error messages to use custom error_code category
@@ -110,7 +136,7 @@ public:
 		//   3.1 send SOCKS5 authentication method message
 		//   3.2 read SOCKS5 authentication response
 		//   3.3 send username+password
-		// 4 send SOCKS command message
+		// 4. send SOCKS command message
 
 		// to avoid unnecessary copying of the handler,
 		// store it in a shaed_ptr
@@ -140,6 +166,7 @@ private:
 	// proxy authentication
 	std::string m_user;
 	std::string m_password;
+	std::string m_dst_name;
 	int m_version;
 	int m_command;
 	// set to one when we're waiting for the
