@@ -34,18 +34,17 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_STORAGE_HPP_INCLUDE
 
 #include <vector>
-#include <bitset>
+#include <sys/types.h>
 
 #ifdef _MSC_VER
 #pragma warning(push, 1)
 #endif
 
-#include <boost/function.hpp>
+#include <boost/function/function2.hpp>
 #include <boost/limits.hpp>
-#include <boost/thread.hpp>
 #include <boost/shared_ptr.hpp>
+#include <boost/scoped_ptr.hpp>
 #include <boost/intrusive_ptr.hpp>
-#include <boost/filesystem/path.hpp>
 
 #ifdef _MSC_VER
 #pragma warning(pop)
@@ -60,49 +59,38 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/config.hpp"
 #include "libtorrent/file.hpp"
 #include "libtorrent/disk_buffer_holder.hpp"
+#include "libtorrent/thread.hpp"
+#include "libtorrent/storage_defs.hpp"
+#include "libtorrent/allocator.hpp"
 
 namespace libtorrent
 {
-	namespace aux
-	{
-		struct piece_checker_data;
-	}
-
-	namespace fs = boost::filesystem;
-
 	class session;
 	struct file_pool;
 	struct disk_io_job;
 	struct disk_buffer_pool;
 	struct session_settings;
 
-	enum storage_mode_t
-	{
-		storage_mode_allocate = 0,
-		storage_mode_sparse,
-		storage_mode_compact
-	};
-	
-	TORRENT_EXPORT std::vector<std::pair<size_type, std::time_t> > get_filesizes(
+	TORRENT_EXTRA_EXPORT std::vector<std::pair<size_type, std::time_t> > get_filesizes(
 		file_storage const& t
-		, fs::path p);
+		, std::string const& p);
 
-	TORRENT_EXPORT bool match_filesizes(
+	TORRENT_EXTRA_EXPORT bool match_filesizes(
 		file_storage const& t
-		, fs::path p
+		, std::string const& p
 		, std::vector<std::pair<size_type, std::time_t> > const& sizes
 		, bool compact_mode
 		, std::string* error = 0);
-
-	struct TORRENT_EXPORT file_allocation_failed: std::exception
+/*
+	struct TORRENT_EXTRA_EXPORT file_allocation_failed: std::exception
 	{
 		file_allocation_failed(const char* error_msg): m_msg(error_msg) {}
 		virtual const char* what() const throw() { return m_msg.c_str(); }
 		virtual ~file_allocation_failed() throw() {}
 		std::string m_msg;
 	};
-
-	struct TORRENT_EXPORT partial_hash
+*/
+	struct TORRENT_EXTRA_EXPORT partial_hash
 	{
 		partial_hash(): offset(0) {}
 		// the number of bytes in the piece that has been hashed
@@ -126,6 +114,7 @@ namespace libtorrent
 		virtual int readv(file::iovec_t const* bufs, int slot, int offset, int num_bufs);
 		virtual int writev(file::iovec_t const* bufs, int slot, int offset, int num_bufs);
 
+		virtual void hint_read(int slot, int offset, int len) {}
 		// negative return value indicates an error
 		virtual int read(char* buf, int slot, int offset, int size) = 0;
 
@@ -140,7 +129,7 @@ namespace libtorrent
 		virtual int sparse_end(int start) const { return start; }
 
 		// non-zero return value indicates an error
-		virtual bool move_storage(fs::path save_path) = 0;
+		virtual bool move_storage(std::string const& save_path) = 0;
 
 		// verify storage dependent fast resume entries
 		virtual bool verify_resume_data(lazy_entry const& rd, error_code& error) = 0;
@@ -171,18 +160,16 @@ namespace libtorrent
 		// non-zero return value indicates an error
 		virtual bool delete_files() = 0;
 
+		virtual void finalize_file(int file) {}
+
 		disk_buffer_pool* disk_pool() { return m_disk_pool; }
 		session_settings const& settings() const { return *m_settings; }
 
-		void set_error(boost::filesystem::path const& file, error_code const& ec) const
-		{
-			m_error_file = file.string();
-			m_error = ec;
-		}
+		void set_error(std::string const& file, error_code const& ec) const;
 
 		error_code const& error() const { return m_error; }
 		std::string const& error_file() const { return m_error_file; }
-		void clear_error() { m_error = error_code(); m_error_file.clear(); }
+		virtual void clear_error() { m_error = error_code(); m_error_file.resize(0); }
 
 		mutable error_code m_error;
 		mutable std::string m_error_file;
@@ -193,18 +180,111 @@ namespace libtorrent
 		session_settings* m_settings;
 	};
 
-	typedef storage_interface* (*storage_constructor_type)(
-		file_storage const&, file_storage const*, fs::path const&, file_pool&);
+	class TORRENT_EXPORT default_storage : public storage_interface, boost::noncopyable
+	{
+	public:
+		default_storage(file_storage const& fs, file_storage const* mapped, std::string const& path
+			, file_pool& fp, std::vector<boost::uint8_t> const& file_prio);
+		~default_storage();
 
-	TORRENT_EXPORT storage_interface* default_storage_constructor(
-		file_storage const&, file_storage const* mapped, fs::path const&, file_pool&);
+		void finalize_file(int file);
+		bool has_any_file();
+		bool rename_file(int index, std::string const& new_filename);
+		bool release_files();
+		bool delete_files();
+		bool initialize(bool allocate_files);
+		bool move_storage(std::string const& save_path);
+		int read(char* buf, int slot, int offset, int size);
+		int write(char const* buf, int slot, int offset, int size);
+		int sparse_end(int start) const;
+		void hint_read(int slot, int offset, int len);
+		int readv(file::iovec_t const* bufs, int slot, int offset, int num_bufs);
+		int writev(file::iovec_t const* buf, int slot, int offset, int num_bufs);
+		size_type physical_offset(int slot, int offset);
+		bool move_slot(int src_slot, int dst_slot);
+		bool swap_slots(int slot1, int slot2);
+		bool swap_slots3(int slot1, int slot2, int slot3);
+		bool verify_resume_data(lazy_entry const& rd, error_code& error);
+		bool write_resume_data(entry& rd) const;
 
-	TORRENT_EXPORT storage_interface* disabled_storage_constructor(
-		file_storage const&, file_storage const* mapped, fs::path const&, file_pool&);
+		// this identifies a read or write operation
+		// so that default_storage::readwritev() knows what to
+		// do when it's actually touching the file
+		struct fileop
+		{
+			size_type (file::*regular_op)(size_type file_offset
+				, file::iovec_t const* bufs, int num_bufs, error_code& ec);
+			size_type (default_storage::*unaligned_op)(boost::intrusive_ptr<file> const& f
+				, size_type file_offset, file::iovec_t const* bufs, int num_bufs
+				, error_code& ec);
+			int cache_setting;
+			int mode;
+		};
+
+		void delete_one_file(std::string const& p);
+		int readwritev(file::iovec_t const* bufs, int slot, int offset
+			, int num_bufs, fileop const&);
+
+		size_type read_unaligned(boost::intrusive_ptr<file> const& file_handle
+			, size_type file_offset, file::iovec_t const* bufs, int num_bufs, error_code& ec);
+		size_type write_unaligned(boost::intrusive_ptr<file> const& file_handle
+			, size_type file_offset, file::iovec_t const* bufs, int num_bufs, error_code& ec);
+
+		file_storage const& files() const { return m_mapped_files?*m_mapped_files:m_files; }
+
+		boost::scoped_ptr<file_storage> m_mapped_files;
+		file_storage const& m_files;
+
+		// helper function to open a file in the file pool with the right mode
+		boost::intrusive_ptr<file> open_file(file_storage::iterator fe, int mode
+			, error_code& ec) const;
+
+		std::vector<boost::uint8_t> m_file_priority;
+		std::string m_save_path;
+		// the file pool is typically stored in
+		// the session, to make all storage
+		// instances use the same pool
+		file_pool& m_pool;
+
+		int m_page_size;
+		bool m_allocate_files;
+	};
+
+	// this storage implementation does not write anything to disk
+	// and it pretends to read, and just leaves garbage in the buffers
+	// this is useful when simulating many clients on the same machine
+	// or when running stress tests and want to take the cost of the
+	// disk I/O out of the picture. This cannot be used for any kind
+	// of normal bittorrent operation, since it will just send garbage
+	// to peers and throw away all the data it downloads. It would end
+	// up being banned immediately
+	class disabled_storage : public storage_interface, boost::noncopyable
+	{
+	public:
+		disabled_storage(int piece_size) : m_piece_size(piece_size) {}
+		bool has_any_file() { return false; }
+		bool rename_file(int index, std::string const& new_filename) { return false; }
+		bool release_files() { return false; }
+		bool delete_files() { return false; }
+		bool initialize(bool allocate_files) { return false; }
+		bool move_storage(std::string const& save_path) { return true; }
+		int read(char* buf, int slot, int offset, int size) { return size; }
+		int write(char const* buf, int slot, int offset, int size) { return size; }
+		size_type physical_offset(int slot, int offset) { return 0; }
+		int readv(file::iovec_t const* bufs, int slot, int offset, int num_bufs);
+		int writev(file::iovec_t const* bufs, int slot, int offset, int num_bufs);
+		bool move_slot(int src_slot, int dst_slot) { return false; }
+		bool swap_slots(int slot1, int slot2) { return false; }
+		bool swap_slots3(int slot1, int slot2, int slot3) { return false; }
+		bool verify_resume_data(lazy_entry const& rd, error_code& error) { return false; }
+		bool write_resume_data(entry& rd) const { return false; }
+
+		int m_piece_size;
+	};
 
 	struct disk_io_thread;
 
-	class TORRENT_EXPORT piece_manager
+	class TORRENT_EXTRA_EXPORT piece_manager
 		: public intrusive_ptr_base<piece_manager>
 		, boost::noncopyable
 	{
@@ -215,38 +295,45 @@ namespace libtorrent
 		piece_manager(
 			boost::shared_ptr<void> const& torrent
 			, boost::intrusive_ptr<torrent_info const> info
-			, fs::path const& path
+			, std::string const& path
 			, file_pool& fp
 			, disk_io_thread& io
 			, storage_constructor_type sc
-			, storage_mode_t sm);
+			, storage_mode_t sm
+			, std::vector<boost::uint8_t> const& file_prio);
 
 		~piece_manager();
 
 		boost::intrusive_ptr<torrent_info const> info() const { return m_info; }
 		void write_resume_data(entry& rd) const;
 
+		void async_finalize_file(int file);
+
 		void async_check_fastresume(lazy_entry const* resume_data
 			, boost::function<void(int, disk_io_job const&)> const& handler);
 		
 		void async_check_files(boost::function<void(int, disk_io_job const&)> const& handler);
 
-		int queued_bytes() const;
-
 		void async_rename_file(int index, std::string const& name
 			, boost::function<void(int, disk_io_job const&)> const& handler);
 
 		void async_read(
 			peer_request const& r
 			, boost::function<void(int, disk_io_job const&)> const& handler
-			, int priority = 0);
+			, int cache_line_size = 0
+			, int cache_expiry = 0);
 
 		void async_read_and_hash(
 			peer_request const& r
 			, boost::function<void(int, disk_io_job const&)> const& handler
-			, int priority = 0);
+			, int cache_expiry = 0);
 
-		void async_write(
+		void async_cache(int piece
+			, boost::function<void(int, disk_io_job const&)> const& handler
+			, int cache_expiry = 0);
+
+		// returns the write queue size
+		int async_write(
 			peer_request const& r
 			, disk_buffer_holder& buffer
 			, boost::function<void(int, disk_io_job const&)> const& f);
@@ -267,7 +354,7 @@ namespace libtorrent
 			boost::function<void(int, disk_io_job const&)> const& handler
 			= boost::function<void(int, disk_io_job const&)>());
 
-		void async_move_storage(fs::path const& p
+		void async_move_storage(std::string const& p
 			, boost::function<void(int, disk_io_job const&)> const& handler);
 
 		void async_save_resume_data(
@@ -286,7 +373,7 @@ namespace libtorrent
 
 	private:
 
-		fs::path save_path() const;
+		std::string save_path() const;
 
 		bool verify_resume_data(lazy_entry const& rd, error_code& e)
 		{ return m_storage->verify_resume_data(rd, e); }
@@ -317,14 +404,16 @@ namespace libtorrent
 		// this function returns true if the checking is complete
 		int check_files(int& current_slot, int& have_piece, error_code& error);
 
+#ifndef TORRENT_NO_DEPRECATE
 		bool compact_allocation() const
 		{ return m_storage_mode == storage_mode_compact; }
+#endif
 
 #ifdef TORRENT_DEBUG
 		std::string name() const { return m_info->name(); }
 #endif
 
-		bool allocate_slots(int num_slots, bool abort_on_disk = false);
+		bool allocate_slots_impl(int num_slots, mutex::scoped_lock& l, bool abort_on_disk = false);
 
 		// updates the ph.h hasher object with the data at the given slot
 		// and optionally a 'small hash' as well, the hash for
@@ -332,6 +421,8 @@ namespace libtorrent
 		int hash_for_slot(int slot, partial_hash& h, int piece_size
 			, int small_piece_size = 0, sha1_hash* small_hash = 0);
 
+		void hint_read_impl(int piece_index, int offset, int size);
+
 		int read_impl(
 			file::iovec_t* bufs
 			, int piece_index
@@ -346,6 +437,8 @@ namespace libtorrent
 
 		size_type physical_offset(int piece_index, int offset);
 
+		void finalize_file(int index);
+
 		// returns the number of pieces left in the
 		// file currently being checked
 		int skip_file() const;
@@ -357,14 +450,14 @@ namespace libtorrent
 			, int current_slot);
 
 		void switch_to_full_mode();
-		sha1_hash hash_for_piece_impl(int piece);
+		sha1_hash hash_for_piece_impl(int piece, int* readback = 0);
 
 		int release_files_impl() { return m_storage->release_files(); }
 		int delete_files_impl() { return m_storage->delete_files(); }
 		int rename_file_impl(int index, std::string const& new_filename)
 		{ return m_storage->rename_file(index, new_filename); }
 
-		int move_storage_impl(fs::path const& save_path);
+		int move_storage_impl(std::string const& save_path);
 
 		int allocate_slot_for_piece(int piece_index);
 #ifdef TORRENT_DEBUG
@@ -404,9 +497,9 @@ namespace libtorrent
 		// it can either be 'unassigned' or 'unallocated'
 		std::vector<int> m_slot_to_piece;
 
-		fs::path m_save_path;
+		std::string m_save_path;
 
-		mutable boost::recursive_mutex m_mutex;
+		mutable mutex m_mutex;
 
 		enum {
 			// the default initial state
@@ -426,8 +519,8 @@ namespace libtorrent
 		// used to move pieces while expanding
 		// the storage from compact allocation
 		// to full allocation
-		disk_buffer_holder m_scratch_buffer;
-		disk_buffer_holder m_scratch_buffer2;
+		aligned_holder m_scratch_buffer;
+		aligned_holder m_scratch_buffer2;
 		// the piece that is in the scratch buffer
 		int m_scratch_piece;
 
