@@ -359,16 +359,9 @@ namespace libtorrent
 
 		// add or remove a url that will be attempted for
 		// finding the file(s) in this torrent.
-		void add_web_seed(std::string const& url, web_seed_entry::type_t type)
-		{
-			m_web_seeds.push_back(web_seed_entry(url, type));
-		}
-
+		void add_web_seed(std::string const& url, web_seed_entry::type_t type);
 		void add_web_seed(std::string const& url, web_seed_entry::type_t type
-			, std::string const& auth, web_seed_entry::headers_t const& extra_headers)
-		{
-			m_web_seeds.push_back(web_seed_entry(url, type, auth, extra_headers));
-		}
+			, std::string const& auth, web_seed_entry::headers_t const& extra_headers);
 	
 		void remove_web_seed(std::string const& url, web_seed_entry::type_t type);
 		void disconnect_web_seed(peer_connection* p);
@@ -712,7 +705,7 @@ namespace libtorrent
 
 		// LOGGING
 #if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING || defined TORRENT_ERROR_LOGGING
-		virtual void debug_log(const std::string& line);
+		virtual void debug_log(const char* fmt, ...) const;
 #endif
 
 		// DEBUG
@@ -1001,6 +994,7 @@ namespace libtorrent
 		};
 
 		// this list is sorted by time_critical_piece::deadline
+		// TODO: this should be a deque
 		std::list<time_critical_piece> m_time_critical_pieces;
 
 		std::string m_trackerid;
