@@ -167,7 +167,7 @@ namespace libtorrent
 		union_endpoint m_tracker_address;
 
 #if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING || defined TORRENT_ERROR_LOGGING
-		virtual void debug_log(const std::string& line) = 0;
+		virtual void debug_log(const char* fmt, ...) const = 0;
 #else
 	private:
 #endif
@@ -224,7 +224,7 @@ namespace libtorrent
 			, io_service& ios
 			, boost::weak_ptr<request_callback> r);
 
-		boost::shared_ptr<request_callback> requester();
+		boost::shared_ptr<request_callback> requester() const;
 		virtual ~tracker_connection() {}
 
 		tracker_request const& tracker_req() const { return m_req; }
