@@ -414,7 +414,7 @@ namespace libtorrent
 		if (is_queued()) p.flags |= peer_info::queued;
 
 		p.client = m_client_version;
-		p.connection_type = get_socket()->get<utp_stream>()
+		p.connection_type = is_utp(*get_socket())
 			? peer_info::bittorrent_utp
 			: peer_info::standard_bittorrent;
 	}
@@ -1551,7 +1551,7 @@ namespace libtorrent
 				error_code ec;
 				char const* err_msg[] = {"no such peer", "not connected", "no support", "no self"};
 				peer_log("<== HOLEPUNCH [ msg:failed error: %d msg: %s ]", error
-					, ((error >= 0 && error < 4)?err_msg[error]:"unknown message id"));
+					, ((error > 0 && error < 5)?err_msg[error-1]:"unknown message id"));
 #endif
 				// #error deal with holepunch errors
 				(void)error;
