@@ -84,87 +84,6 @@ namespace libtorrent
 		return ret;
 	}
 
-	bool is_alpha(char c)
-	{
-		return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z');
-	}
-
-	bool is_digit(char c)
-	{
-		return c >= '0' && c <= '9';
-	}
-
-	bool is_print(char c)
-	{
-		return c >= 32 && c < 127;
-	}
-
-	bool is_space(char c)
-	{
-		const static char* ws = " \t\n\r\f\v";
-		return std::strchr(ws, c) != 0;
-	}
-
-	// generate a url-safe random string
-	void url_random(char* begin, char* end)
-	{
-		// http-accepted characters:
-		// excluding ', since some buggy trackers don't support that
-		static char const printable[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-			"abcdefghijklmnopqrstuvwxyz-_.!~*()";
-
-		// the random number
-		while (begin != end)
-			*begin++ = printable[random() % (sizeof(printable)-1)];
-	}
-
-	char to_lower(char c)
-	{
-		return (c >= 'A' && c <= 'Z') ? c - 'A' + 'a' : c;
-	}
-
-	int split_string(char const** tags, int buf_size, char* in)
-	{
-		int ret = 0;
-		char* i = in;
-		for (;*i; ++i)
-		{
-			if (!is_print(*i) || is_space(*i))
-			{
-				*i = 0;
-				if (ret == buf_size) return ret;
-				continue;
-			}
-			if (i == in || i[-1] == 0)
-			{
-				tags[ret++] = i;
-			}
-		}
-		return ret;
-	}
-
-	bool string_begins_no_case(char const* s1, char const* s2)
-	{
-		while (*s1 != 0)
-		{
-			if (to_lower(*s1) != to_lower(*s2)) return false;
-			++s1;
-			++s2;
-		}
-		return true;
-	}
-
-	bool string_equal_no_case(char const* s1, char const* s2)
-	{
-		while (to_lower(*s1) == to_lower(*s2))
-		{
-			if (*s1 == 0) return true;
-			++s1;
-			++s2;
-		}
-		return false;
-	}
-
 	std::string unescape_string(std::string const& s, error_code& ec)
 	{
 		std::string ret;
