@@ -43,6 +43,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/scoped_ptr.hpp>
 #include <boost/static_assert.hpp>
 
+#include <sys/stat.h>
+
 #ifdef TORRENT_WINDOWS
 // windows part
 
@@ -57,16 +59,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #endif
 #include <windows.h>
 #include <winioctl.h>
+#ifndef TORRENT_MINGW
 #include <direct.h> // for _getcwd, _mkdir
+#else
+#include <dirent.h>
+#endif
 #include <sys/types.h>
-#include <sys/stat.h>
 #else
 // posix part
 
 #define _FILE_OFFSET_BITS 64
 #include <unistd.h>
 #include <fcntl.h> // for F_LOG2PHYS
-#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/statvfs.h>
 #include <errno.h>
@@ -167,7 +171,7 @@ namespace libtorrent
 		std::string f = convert_to_native(inf);
 #endif
 
-#ifdef TORRENT_WINDOWS
+#if defined TORRENT_WINDOWS
 		struct _stati64 ret;
 #if TORRENT_USE_WSTRING
 		if (_wstati64(f.c_str(), &ret) < 0)
@@ -196,7 +200,20 @@ namespace libtorrent
 		s->atime = ret.st_atime;
 		s->mtime = ret.st_mtime;
 		s->ctime = ret.st_ctime;
-		s->mode = ret.st_mode;
+#if defined TORRENT_WINDOWS
+    s->mode = ((ret.st_mode & _S_IFREG) ? file_status::regular_file : 0)
+      | ((ret.st_mode & _S_IFDIR) ? file_status::directory : 0)
+      | ((ret.st_mode & _S_IFCHR) ? file_status::character_special : 0)
+      | ((ret.st_mode & _S_IFIFO) ? file_status::fifo : 0);
+#else
+    s->mode = (S_ISREG(ret.st_mode) ? file_status::regular_file : 0)
+      | (S_ISDIR(ret.st_mode) ? file_status::directory : 0)
+      | (S_ISLNK(ret.st_mode) ? file_status::link : 0)
+      | (S_ISFIFO(ret.st_mode) ? file_status::fifo : 0)
+      | (S_ISCHR(ret.st_mode) ? file_status::character_special : 0)
+      | (S_ISBLK(ret.st_mode) ? file_status::block_special : 0)
+      | (S_ISSOCK(ret.st_mode) ? file_status::socket : 0);
+#endif
 	}
 
 	void rename(std::string const& inf, std::string const& newf, error_code& ec)
@@ -538,7 +555,7 @@ namespace libtorrent
 
 	std::string current_working_directory()
 	{
-#ifdef TORRENT_WINDOWS
+#if defined TORRENT_WINDOWS && !defined TORRENT_MINGW
 #if TORRENT_USE_WSTRING
 		wchar_t cwd[TORRENT_MAX_PATH];
 		_wgetcwd(cwd, sizeof(cwd) / sizeof(wchar_t));
@@ -550,7 +567,7 @@ namespace libtorrent
 		char cwd[TORRENT_MAX_PATH];
 		if (getcwd(cwd, sizeof(cwd)) == 0) return "/";
 #endif
-#if defined TORRENT_WINDOWS && TORRENT_USE_WSTRING
+#if defined TORRENT_WINDOWS && !defined TORRENT_MINGW && TORRENT_USE_WSTRING
 		return convert_from_wstring(cwd);
 #else
 		return convert_from_native(cwd);
@@ -1189,6 +1206,19 @@ namespace libtorrent
 
 	size_type file::readv(size_type file_offset, iovec_t const* bufs, int num_bufs, error_code& ec)
 	{
+#ifdef TORRENT_WINDOWS
+		if (m_file_handle == INVALID_HANDLE_VALUE)
+		{
+			ec = error_code(ERROR_INVALID_HANDLE, get_system_category());
+			return -1;
+		}
+#else
+		if (m_fd == -1)
+		{
+			ec = error_code(EBADF, get_system_category());
+			return -1;
+		}
+#endif
 		TORRENT_ASSERT((m_open_mode & rw_mask) == read_only || (m_open_mode & rw_mask) == read_write);
 		TORRENT_ASSERT(bufs);
 		TORRENT_ASSERT(num_bufs > 0);
@@ -1291,24 +1321,37 @@ namespace libtorrent
 		ol.OffsetHigh = DWORD(file_offset >> 32);
 		ol.Offset = DWORD(file_offset & 0xffffffff);
 		ol.hEvent = CreateEvent(0, true, false, 0);
+		if (ol.hEvent == NULL)
+		{
+			ec.assign(GetLastError(), get_system_category());
+			return -1;
+		}
 
 		ret += size;
 		size = num_pages * m_page_size;
 		if (ReadFileScatter(m_file_handle, segment_array, size, 0, &ol) == 0)
 		{
 			DWORD last_error = GetLastError();
-			if (last_error != ERROR_IO_PENDING)
+			if (last_error != ERROR_IO_PENDING
+#ifndef TORRENT_MINGW
+				&& last_error != ERROR_CANT_WAIT
+#endif
+)
 			{
-				ec.assign(GetLastError(), get_system_category());
+				ec.assign(last_error, get_system_category());
 				CloseHandle(ol.hEvent);
 				return -1;
 			}
 			DWORD num_read;
 			if (GetOverlappedResult(m_file_handle, &ol, &num_read, true) == 0)
 			{
-				if (GetLastError() != ERROR_HANDLE_EOF)
+				DWORD last_error = GetLastError();
+				if (last_error != ERROR_HANDLE_EOF)
 				{
-					ec.assign(GetLastError(), get_system_category());
+#ifndef TORRENT_MINGW
+					TORRENT_ASSERT(last_error != ERROR_CANT_WAIT);
+#endif
+					ec.assign(last_error, get_system_category());
 					CloseHandle(ol.hEvent);
 					return -1;
 				}
@@ -1400,6 +1443,19 @@ namespace libtorrent
 
 	size_type file::writev(size_type file_offset, iovec_t const* bufs, int num_bufs, error_code& ec)
 	{
+#ifdef TORRENT_WINDOWS
+		if (m_file_handle == INVALID_HANDLE_VALUE)
+		{
+			ec = error_code(ERROR_INVALID_HANDLE, get_system_category());
+			return -1;
+		}
+#else
+		if (m_fd == -1)
+		{
+			ec = error_code(EBADF, get_system_category());
+			return -1;
+		}
+#endif
 		TORRENT_ASSERT((m_open_mode & rw_mask) == write_only || (m_open_mode & rw_mask) == read_write);
 		TORRENT_ASSERT(bufs);
 		TORRENT_ASSERT(num_bufs > 0);
@@ -1493,6 +1549,11 @@ namespace libtorrent
 		ol.OffsetHigh = DWORD(file_offset >> 32);
 		ol.Offset = DWORD(file_offset & 0xffffffff);
 		ol.hEvent = CreateEvent(0, true, false, 0);
+		if (ol.hEvent == NULL)
+		{
+			ec.assign(GetLastError(), get_system_category());
+			return -1;
+		}
 
 		ret += size;
 		size_type file_size = 0;
@@ -1508,17 +1569,26 @@ namespace libtorrent
 
 		if (WriteFileGather(m_file_handle, segment_array, size, 0, &ol) == 0)
 		{
-			if (GetLastError() != ERROR_IO_PENDING)
+			DWORD last_error = GetLastError();
+			if (last_error != ERROR_IO_PENDING
+#ifndef TORRENT_MINGW
+				&& last_error != ERROR_CANT_WAIT
+#endif
+				)
 			{
-				TORRENT_ASSERT(GetLastError() != ERROR_BAD_ARGUMENTS);
-				ec.assign(GetLastError(), get_system_category());
+				TORRENT_ASSERT(last_error != ERROR_BAD_ARGUMENTS);
+				ec.assign(last_error, get_system_category());
 				CloseHandle(ol.hEvent);
 				return -1;
 			}
 			DWORD num_written;
 			if (GetOverlappedResult(m_file_handle, &ol, &num_written, true) == 0)
 			{
-				ec.assign(GetLastError(), get_system_category());
+				DWORD last_error = GetLastError();
+#ifndef TORRENT_MINGW
+				TORRENT_ASSERT(last_error != ERROR_CANT_WAIT);
+#endif
+				ec.assign(last_error, get_system_category());
 				CloseHandle(ol.hEvent);
 				return -1;
 			}
