@@ -33,6 +33,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/pch.hpp"
 
 #include "libtorrent/file_storage.hpp"
+#include "libtorrent/string_util.hpp" // for allocate_string_copy
 #include "libtorrent/file.hpp"
 #include "libtorrent/utf8.hpp"
 #include <boost/bind.hpp>
@@ -147,7 +148,7 @@ namespace libtorrent
 		}
 		else
 		{
-			name = borrow_chars ? n : strdup(n);
+			name = borrow_chars ? n : allocate_string_copy(n);
 		}
 		name_len = borrow_chars;
 	}
@@ -282,7 +283,7 @@ namespace libtorrent
 	{
 		TORRENT_ASSERT(file_index < num_files());
 		TORRENT_ASSERT(file_index >= 0);
-		size_type offset = file_offset + at(file_index).offset;
+		size_type offset = file_offset + internal_at(file_index).offset;
 
 		peer_request ret;
 		ret.piece = int(offset / piece_length());
