@@ -345,6 +345,7 @@ namespace libtorrent
 
 	policy::policy(torrent* t)
 		: m_torrent(t)
+		, m_locked_peer(NULL)
 		, m_round_robin(0)
 		, m_num_connect_candidates(0)
 		, m_num_seeds(0)
@@ -366,6 +367,12 @@ namespace libtorrent
 				++i;
 				continue;
 			}
+
+			if (*i == m_locked_peer)
+			{
+				++i;
+				continue;
+			}
 		
 			if (ses.m_alerts.should_post<peer_blocked_alert>())
 				ses.m_alerts.post_alert(peer_blocked_alert(m_torrent->get_handle(), (*i)->address()));
@@ -418,6 +425,7 @@ namespace libtorrent
 	{
 		INVARIANT_CHECK;
 		TORRENT_ASSERT(i != m_peers.end());
+		TORRENT_ASSERT(m_locked_peer != *i);
 
 		if (m_torrent->has_picker())
 			m_torrent->picker().clear_peer(*i);
@@ -468,12 +476,14 @@ namespace libtorrent
 	bool policy::should_erase_immediately(peer const& p) const
 	{
 		TORRENT_ASSERT(p.in_use);
+		if (&p == m_locked_peer) return false;
 		return p.source == peer_info::resume_data;
 	}
 
 	bool policy::is_erase_candidate(peer const& pe, bool finished) const
 	{
 		TORRENT_ASSERT(pe.in_use);
+		if (&pe == m_locked_peer) return false;
 		if (pe.connection) return false;
 		if (is_connect_candidate(pe, finished)) return false;
 
@@ -484,6 +494,7 @@ namespace libtorrent
 	bool policy::is_force_erase_candidate(peer const& pe) const
 	{
 		TORRENT_ASSERT(pe.in_use);
+		if (&pe == m_locked_peer) return false;
 		return pe.connection == 0;
 	}
 
@@ -766,28 +777,6 @@ namespace libtorrent
 		TORRENT_ASSERT(c.remote() == c.get_socket()->remote_endpoint(ec) || ec);
 		TORRENT_ASSERT(!m_torrent->is_paused());
 
-		aux::session_impl& ses = m_torrent->session();
-		
-		if (m_torrent->num_peers() >= m_torrent->max_connections()
-			&& ses.num_connections() >= ses.settings().connections_limit
-			&& c.remote().address() != m_torrent->current_tracker().address())
-		{
-#if defined TORRENT_LOGGING || defined TORRENT_VERBOSE_LOGGING
-			(*m_torrent->session().m_logger) << time_now_string()
-				<< " *** TOO MANY CONNECTIONS ["
-				" torrent: " << m_torrent->name() <<
-				" torrent peers: " << m_torrent->num_peers() <<
-				" torrent limit: " << m_torrent->max_connections() <<
-				" global peers: " << ses.num_connections() <<
-				" global limit: " << ses.settings().connections_limit <<
-				" global list peers " << int(m_peers.size()) <<
-				" global list limit: " << m_torrent->settings().max_peerlist_size <<
-				" ]\n";
-#endif
-			c.disconnect(errors::too_many_connections);
-			return false;
-		}
-
 #if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING
 		if (c.remote().address() == m_torrent->current_tracker().address())
 		{
@@ -805,7 +794,11 @@ namespace libtorrent
 			std::pair<iterator, iterator> range = find_peers(remote.address());
 			iter = std::find_if(range.first, range.second, match_peer_endpoint(remote));
 	
-			if (iter != range.second) found = true;
+			if (iter != range.second)
+			{
+				TORRENT_ASSERT((*iter)->in_use);
+				found = true;
+			}
 		}
 		else
 		{
@@ -814,7 +807,11 @@ namespace libtorrent
 				, c.remote().address(), peer_address_compare()
 			);
 
-			if (iter != m_peers.end() && (*iter)->address() == c.remote().address()) found = true;
+			if (iter != m_peers.end() && (*iter)->address() == c.remote().address())
+			{
+				TORRENT_ASSERT((*iter)->in_use);
+				found = true;
+			}
 		}
 
 		// make sure the iterator we got is properly sorted relative
@@ -824,12 +821,14 @@ namespace libtorrent
 //			|| (iter != m_peers.end() && c.remote().address() < (*iter)->address())
 //			|| (iter != m_peers.end() && iter != m_peers.begin() && (*(iter-1))->address() < c.remote().address()));
 
+		aux::session_impl& ses = m_torrent->session();
+
 		if (found)
 		{
 			i = *iter;
+			TORRENT_ASSERT(i->in_use);
 			TORRENT_ASSERT(i->connection != &c);
 
-			TORRENT_ASSERT(i->in_use);
 			if (i->banned)
 			{
 				c.disconnect(errors::peer_banned);
@@ -868,23 +867,78 @@ namespace libtorrent
 				// or the current one is already connected
 				if (ec2)
 				{
+					TORRENT_ASSERT(m_locked_peer == NULL);
+					m_locked_peer = i;
 					i->connection->disconnect(ec2);
 					TORRENT_ASSERT(i->connection == 0);
+					m_locked_peer = NULL;
 				}
-				else if (!i->connection->is_connecting() || c.is_outgoing())
+				else if (i->connection->is_outgoing() == c.is_outgoing())
 				{
+					// if the other end connected to us both times, just drop
+					// the second one. Or if we made both connections.
 					c.disconnect(errors::duplicate_peer_id);
 					return false;
 				}
 				else
 				{
-#if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING
-					m_torrent->debug_log("duplicate connection. existing connection"
-					" is connecting and this connection is incoming. closing existing "
-					"connection in favour of this one");
+					// at this point, we need to disconnect either
+					// i->connection or c. In order for both this client
+					// and the client on the other end to decide to
+					// disconnect the same one, we need a consistent rule to
+					// select which one.
+
+					bool outgoing1 = c.is_outgoing();
+
+					// for this, we compare our endpoints (IP and port)
+					// and whoever has the lower IP,port should be the
+					// one keeping its outgoing connection. Since outgoing
+					// ports are selected at random by the OS, we need
+					// to be careful to only look at the target end of a
+					// connection for the endpoint.
+
+					tcp::endpoint our_ep = outgoing1 ? other_socket->local_endpoint(ec1) : this_socket->local_endpoint(ec1);
+					tcp::endpoint other_ep = outgoing1 ? this_socket->remote_endpoint(ec1) : other_socket->remote_endpoint(ec1);
+
+					if (our_ep < other_ep)
+					{
+#ifdef TORRENT_VERBOSE_LOGGING
+						c.peer_log("*** DUPLICATE PEER RESOLUTION [ \"%s\" < \"%s\" ]"
+							, print_endpoint(our_ep).c_str(), print_endpoint(other_ep).c_str());
+						i->connection->peer_log("*** DUPLICATE PEER RESOLUTION [ \"%s\" < \"%s\" ]"
+							, print_endpoint(our_ep).c_str(), print_endpoint(other_ep).c_str());
 #endif
-					i->connection->disconnect(errors::duplicate_peer_id);
-					TORRENT_ASSERT(i->connection == 0);
+
+						// we should keep our outgoing connection
+						if (!outgoing1)
+						{
+							c.disconnect(errors::duplicate_peer_id);
+							return false;
+						}
+						TORRENT_ASSERT(m_locked_peer == NULL);
+						m_locked_peer = i;
+						i->connection->disconnect(errors::duplicate_peer_id);
+						m_locked_peer = NULL;
+					}
+					else
+					{
+#ifdef TORRENT_VERBOSE_LOGGING
+						c.peer_log("*** DUPLICATE PEER RESOLUTION [ \"%s\" >= \"%s\" ]"
+							, print_endpoint(our_ep).c_str(), print_endpoint(other_ep).c_str());
+						i->connection->peer_log("*** DUPLICATE PEER RESOLUTION [ \"%s\" >= \"%s\" ]"
+							, print_endpoint(our_ep).c_str(), print_endpoint(other_ep).c_str());
+#endif
+						// they should keep their outgoing connection
+						if (outgoing1)
+						{
+							c.disconnect(errors::duplicate_peer_id);
+							return false;
+						}
+						TORRENT_ASSERT(m_locked_peer == NULL);
+						m_locked_peer = i;
+						i->connection->disconnect(errors::duplicate_peer_id);
+						m_locked_peer = NULL;
+					}
 				}
 			}
 
@@ -1036,7 +1090,10 @@ namespace libtorrent
 					// we need to make sure we don't let it do that, by unlinking
 					// the peer_connection from the policy::peer first.
 					p->connection->set_peer_info(0);
+					TORRENT_ASSERT(m_locked_peer == NULL);
+					m_locked_peer = p;
 					p->connection->disconnect(errors::duplicate_peer_id);
+					m_locked_peer = NULL;
 					erase_peer(p);
 					return false;
 				}
@@ -1283,6 +1340,14 @@ namespace libtorrent
 		if (remote.address() == address() || remote.port() == 0)
 			return 0;
 
+#if TORRENT_USE_IPV6
+		// don't allow link-local IPv6 addresses since they
+		// can't be used like normal addresses, they require an interface
+		// and will just cause connect() to fail with EINVAL
+		if (remote.address().is_v6() && remote.address().to_v6().is_link_local())
+			return 0;
+#endif
+
 		aux::session_impl& ses = m_torrent->session();
 
 		// if this is an i2p torrent, and we don't allow mixed mode
@@ -1504,7 +1569,8 @@ namespace libtorrent
 		// to avoid adding one entry for every single connection
 		// the peer makes to us, don't save this entry
 		if (m_torrent->settings().allow_multiple_connections_per_ip
-			&& !p->connectable)
+			&& !p->connectable
+			&& p != m_locked_peer)
 		{
 			erase_peer(p);
 		}
