@@ -188,7 +188,7 @@ namespace libtorrent
 
 		// we will probably see a high rate of alerts, make it less
 		// likely to loose alerts
-		set.alert_queue_size = 10000;
+		set.alert_queue_size = 50000;
 
 		// allow 500 files open at a time
 		set.file_pool_size = 500;
@@ -1167,7 +1167,7 @@ namespace libtorrent
 		, active_downloads(3)
 		, active_seeds(5)
 		, active_dht_limit(88) // don't announce more than once every 40 seconds
-		, active_tracker_limit(360) // don't announce to trackers more than once every 5 seconds
+		, active_tracker_limit(1600) // don't announce to trackers more than once every 1.125 seconds
 		, active_lsd_limit(60) // don't announce to local network more than once every 5 seconds
 		, active_limit(15)
 		, auto_manage_prefer_seeds(false)
@@ -1267,7 +1267,7 @@ namespace libtorrent
 		, torrent_connect_boost(10)
 		, seeding_outgoing_connections(true)
 		, no_connect_privileged_ports(true)
-		, alert_queue_size(1000)
+		, alert_queue_size(6000)
 		, max_metadata_size(3*1024*1024)
 		, smooth_connects(true)
 		, always_send_user_agent(false)
