@@ -969,6 +969,11 @@ namespace libtorrent
 		TORRENT_ASYNC_CALL1(reset_piece_deadline, index);
 	}
 
+	boost::shared_ptr<torrent> torrent_handle::native_handle() const
+	{
+		return m_torrent.lock();
+	}
+
 	std::size_t hash_value(torrent_status const& ts)
 	{
 		return hash_value(ts.handle);
@@ -978,5 +983,6 @@ namespace libtorrent
 	{
 		return std::size_t(th.m_torrent.lock().get());
 	}
+
 }
 
