@@ -156,7 +156,7 @@ namespace libtorrent
 		, m_req(req)
 	{}
 
-	boost::shared_ptr<request_callback> tracker_connection::requester()
+	boost::shared_ptr<request_callback> tracker_connection::requester() const
 	{
 		return m_requester.lock();
 	}
@@ -323,7 +323,7 @@ namespace libtorrent
 
 #if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING || defined TORRENT_ERROR_LOGGING
 			boost::shared_ptr<request_callback> rc = c->requester();
-			if (rc) rc->debug_log("aborting: " + req.url);
+			if (rc) rc->debug_log("aborting: %s", req.url.c_str());
 #endif
 		}
 		l.unlock();
