@@ -34,9 +34,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket.hpp"
 #include "libtorrent/udp_socket.hpp"
 #include "libtorrent/connection_queue.hpp"
-#include "libtorrent/escape_string.hpp"
 #include "libtorrent/socket_io.hpp"
 #include "libtorrent/error.hpp"
+#include "libtorrent/string_util.hpp" // for allocate_string_copy
 #include <stdlib.h>
 #include <boost/bind.hpp>
 #include <boost/array.hpp>
@@ -86,6 +86,11 @@ udp_socket::udp_socket(asio::io_service& ios
 	m_magic = 0x1337;
 	m_started = false;
 	m_outstanding_when_aborted = -1;
+	m_outstanding_connect_queue = 0;
+	m_outstanding_connect = 0;
+	m_outstanding_timeout = 0;
+	m_outstanding_resolve = 0;
+	m_outstanding_socks = 0;
 #if defined BOOST_HAS_PTHREADS
 	m_thread = 0;
 #endif
@@ -152,7 +157,7 @@ void udp_socket::send_hostname(char const* hostname, int port
 	m_queue.push_back(queued_packet());
 	queued_packet& qp = m_queue.back();
 	qp.ep.port(port);
-	qp.hostname = strdup(hostname);
+	qp.hostname = allocate_string_copy(hostname);
 	qp.buf.insert(qp.buf.begin(), p, p + len);
 	qp.flags = 0;
 }
@@ -303,9 +308,17 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 			&& e != asio::error::connection_refused
 			&& e != asio::error::connection_aborted
 			&& e != asio::error::operation_aborted
+			&& e != asio::error::network_reset
+			&& e != asio::error::network_unreachable
 #ifdef WIN32
 			// ERROR_MORE_DATA means the same thing as EMSGSIZE
 			&& e != error_code(ERROR_MORE_DATA, get_system_category())
+			&& e != error_code(ERROR_HOST_UNREACHABLE, get_system_category())
+			&& e != error_code(ERROR_PORT_UNREACHABLE, get_system_category())
+			&& e != error_code(ERROR_RETRY, get_system_category())
+			&& e != error_code(ERROR_NETWORK_UNREACHABLE, get_system_category())
+			&& e != error_code(ERROR_CONNECTION_REFUSED, get_system_category())
+			&& e != error_code(ERROR_CONNECTION_ABORTED, get_system_category())
 #endif
 			&& e != asio::error::message_size)
 		{
@@ -560,8 +573,18 @@ void udp_socket::close()
 		// we just called done, which means on_timeout
 		// won't be called. Decrement the outstanding
 		// ops counter for that
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+		TORRENT_ASSERT(m_outstanding_timeout > 0);
+		--m_outstanding_timeout;
+#endif
 		TORRENT_ASSERT(m_outstanding_ops > 0);
 		--m_outstanding_ops;
+		TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+			+ m_outstanding_timeout
+			+ m_outstanding_resolve
+			+ m_outstanding_connect_queue
+			+ m_outstanding_socks);
+
 		if (m_abort)
 		{
 			maybe_clear_callback();
@@ -621,8 +644,10 @@ void udp_socket::bind(udp::endpoint const& ep, error_code& ec)
 #if TORRENT_USE_IPV6
 	else
 	{
+#ifdef IPV6_V6ONLY
 		m_ipv6_sock.set_option(v6only(true), ec);
 		if (ec) return;
+#endif
 		m_ipv6_sock.bind(ep, ec);
 		if (ec) return;
 		if (m_v6_outstanding == 0)
@@ -685,7 +710,9 @@ void udp_socket::bind(int port)
 #if defined TORRENT_ASIO_DEBUGGING
 		add_outstanding_async("udp_socket::on_read");
 #endif
+#ifdef IPV6_V6ONLY
 		m_ipv6_sock.set_option(v6only(true), ec);
+#endif
 		m_ipv6_sock.bind(udp::endpoint(address_v6::any(), port), ec);
 
 		if (m_v6_outstanding == 0)
@@ -724,6 +751,9 @@ void udp_socket::set_proxy_settings(proxy_settings const& ps)
 		// connect to socks5 server and open up the UDP tunnel
 		tcp::resolver::query q(ps.hostname, to_string(ps.port).elems);
 		++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+		++m_outstanding_resolve;
+#endif
 		m_resolver.async_resolve(q, boost::bind(
 			&udp_socket::on_name_lookup, this, _1, _2));
 	}
@@ -731,8 +761,18 @@ void udp_socket::set_proxy_settings(proxy_settings const& ps)
 
 void udp_socket::on_name_lookup(error_code const& e, tcp::resolver::iterator i)
 {
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_resolve > 0);
+	--m_outstanding_resolve;
+#endif
+
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
 
 	if (m_abort)
 	{
@@ -762,15 +802,36 @@ void udp_socket::on_name_lookup(error_code const& e, tcp::resolver::iterator i)
 	// when m_outstanding_ops may be decremented
 	// To simplyfy this, it's probably a good idea to
 	// merge on_connect and on_timeout to a single function
-	++m_outstanding_ops;
+
+	// on_timeout may be called before on_connected
+	// so increment the outstanding ops
+	// it may also not be called in case we call
+	// connection_queue::done first, so be sure to
+	// decrement if that happens
+	m_outstanding_ops += 2;
+
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_timeout;
+	++m_outstanding_connect_queue;
+#endif
 	m_cc.enqueue(boost::bind(&udp_socket::on_connect, this, _1)
 		, boost::bind(&udp_socket::on_timeout, this), seconds(10));
 }
 
 void udp_socket::on_timeout()
 {
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_timeout > 0);
+	--m_outstanding_timeout;
+#endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
@@ -787,8 +848,35 @@ void udp_socket::on_timeout()
 void udp_socket::on_connect(int ticket)
 {
 	TORRENT_ASSERT(is_single_thread());
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_connect_queue > 0);
+	--m_outstanding_connect_queue;
+#endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
+	if (ticket == -1)
+	{
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+		TORRENT_ASSERT(m_outstanding_timeout > 0);
+		--m_outstanding_timeout;
+#endif
+		TORRENT_ASSERT(m_outstanding_ops > 0);
+		--m_outstanding_ops;
+		TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+			+ m_outstanding_timeout
+			+ m_outstanding_resolve
+			+ m_outstanding_connect_queue
+			+ m_outstanding_socks);
+		close();
+		return;
+	}
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
@@ -798,26 +886,18 @@ void udp_socket::on_connect(int ticket)
 
 	if (m_abort) return;
 	if (is_closed()) return;
-	if (ticket == -1)
-	{
-		close();
-		return;
-	}
 
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("udp_socket::on_connected");
 #endif
 	m_connection_ticket = ticket;
-	// at this point on_timeout may be called before on_connected
-	// so increment the outstanding ops
-	// it may also not be called in case we call
-	// connection_queue::done first, so be sure to
-	// decrement if that happens
-	++m_outstanding_ops;
 
 	error_code ec;
 	m_socks5_sock.open(m_proxy_addr.address().is_v4()?tcp::v4():tcp::v6(), ec);
 	++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_connect;
+#endif
 	m_socks5_sock.async_connect(tcp::endpoint(m_proxy_addr.address(), m_proxy_addr.port())
 		, boost::bind(&udp_socket::on_connected, this, _1));
 }
@@ -826,15 +906,24 @@ void udp_socket::on_connected(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::on_connected");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_connect > 0);
+	--m_outstanding_connect;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
 		return;
 	}
-
 	CHECK_MAGIC;
 
 	if (e == asio::error::operation_aborted) return;
@@ -846,8 +935,18 @@ void udp_socket::on_connected(error_code const& e)
 	// we just called done, which means on_timeout
 	// won't be called. Decrement the outstanding
 	// ops counter for that
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_timeout > 0);
+	--m_outstanding_timeout;
+#endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
@@ -884,6 +983,9 @@ void udp_socket::on_connected(error_code const& e)
 	add_outstanding_async("udp_socket::on_handshake1");
 #endif
 	++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_socks;
+#endif
 	asio::async_write(m_socks5_sock, asio::buffer(m_tmp_buf, p - m_tmp_buf)
 		, boost::bind(&udp_socket::handshake1, this, _1));
 }
@@ -892,15 +994,24 @@ void udp_socket::handshake1(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::on_handshake1");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_socks > 0);
+	--m_outstanding_socks;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
 		return;
 	}
-
 	CHECK_MAGIC;
 	if (e) return;
 
@@ -910,6 +1021,9 @@ void udp_socket::handshake1(error_code const& e)
 	add_outstanding_async("udp_socket::on_handshake2");
 #endif
 	++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_socks;
+#endif
 	asio::async_read(m_socks5_sock, asio::buffer(m_tmp_buf, 2)
 		, boost::bind(&udp_socket::handshake2, this, _1));
 }
@@ -918,9 +1032,19 @@ void udp_socket::handshake2(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::on_handshake2");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_socks > 0);
+	--m_outstanding_socks;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
@@ -965,6 +1089,9 @@ void udp_socket::handshake2(error_code const& e)
 		add_outstanding_async("udp_socket::on_handshake3");
 #endif
 		++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+		++m_outstanding_socks;
+#endif
 		asio::async_write(m_socks5_sock, asio::buffer(m_tmp_buf, p - m_tmp_buf)
 			, boost::bind(&udp_socket::handshake3, this, _1));
 	}
@@ -980,15 +1107,24 @@ void udp_socket::handshake3(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::on_handshake3");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_socks > 0);
+	--m_outstanding_socks;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
 		return;
 	}
-
 	CHECK_MAGIC;
 	if (e) return;
 
@@ -998,6 +1134,9 @@ void udp_socket::handshake3(error_code const& e)
 	add_outstanding_async("udp_socket::on_handshake4");
 #endif
 	++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_socks;
+#endif
 	asio::async_read(m_socks5_sock, asio::buffer(m_tmp_buf, 2)
 		, boost::bind(&udp_socket::handshake4, this, _1));
 }
@@ -1006,15 +1145,24 @@ void udp_socket::handshake4(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::on_handshake4");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_socks > 0);
+	--m_outstanding_socks;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
 		return;
 	}
-
 	CHECK_MAGIC;
 	if (e) return;
 
@@ -1061,6 +1209,9 @@ void udp_socket::socks_forward_udp()
 	add_outstanding_async("udp_socket::connect1");
 #endif
 	++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_socks;
+#endif
 	asio::async_write(m_socks5_sock, asio::buffer(m_tmp_buf, p - m_tmp_buf)
 		, boost::bind(&udp_socket::connect1, this, _1));
 }
@@ -1069,15 +1220,24 @@ void udp_socket::connect1(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::connect1");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_socks > 0);
+	--m_outstanding_socks;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
 		return;
 	}
-
 	CHECK_MAGIC;
 	if (e) return;
 
@@ -1087,6 +1247,9 @@ void udp_socket::connect1(error_code const& e)
 	add_outstanding_async("udp_socket::connect2");
 #endif
 	++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_socks;
+#endif
 	asio::async_read(m_socks5_sock, asio::buffer(m_tmp_buf, 10)
 		, boost::bind(&udp_socket::connect2, this, _1));
 }
@@ -1095,16 +1258,25 @@ void udp_socket::connect2(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::connect2");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_socks > 0);
+	--m_outstanding_socks;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		m_queue.clear();
 		maybe_clear_callback();
 		return;
 	}
-
 	CHECK_MAGIC;
 	if (e)
 	{
@@ -1165,6 +1337,9 @@ void udp_socket::connect2(error_code const& e)
 	add_outstanding_async("udp_socket::hung_up");
 #endif
 	++m_outstanding_ops;
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	++m_outstanding_socks;
+#endif
 	asio::async_read(m_socks5_sock, asio::buffer(m_tmp_buf, 10)
 		, boost::bind(&udp_socket::hung_up, this, _1));
 }
@@ -1173,15 +1348,24 @@ void udp_socket::hung_up(error_code const& e)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("udp_socket::hung_up");
+#endif
+#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+	TORRENT_ASSERT(m_outstanding_socks > 0);
+	--m_outstanding_socks;
 #endif
 	TORRENT_ASSERT(m_outstanding_ops > 0);
 	--m_outstanding_ops;
+	TORRENT_ASSERT(m_outstanding_ops == m_outstanding_connect
+		+ m_outstanding_timeout
+		+ m_outstanding_resolve
+		+ m_outstanding_connect_queue
+		+ m_outstanding_socks);
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
 		return;
 	}
-  
 	CHECK_MAGIC;
 	TORRENT_ASSERT(is_single_thread());
 
@@ -1196,77 +1380,35 @@ rate_limited_udp_socket::rate_limited_udp_socket(io_service& ios
 	, callback2_t const& c2
 	, connection_queue& cc)
 	: udp_socket(ios, c, c2, cc)
-	, m_timer(ios)
-	, m_queue_size_limit(200)
-	, m_rate_limit(4000)
-	, m_quota(4000)
+	, m_rate_limit(8000)
+	, m_quota(8000)
 	, m_last_tick(time_now())
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("rate_limited_udp_socket::on_tick");
 #endif
-	error_code ec;
-	m_timer.expires_from_now(seconds(1), ec);
-	m_timer.async_wait(boost::bind(&rate_limited_udp_socket::on_tick, this, _1));
-	TORRENT_ASSERT_VAL(!ec, ec);
 }
 
 bool rate_limited_udp_socket::send(udp::endpoint const& ep, char const* p
 	, int len, error_code& ec, int flags)
 {
-	if (m_quota < len)
-	{
-		if (int(m_queue.size()) >= m_queue_size_limit && (flags & dont_drop) == 0)
-			return false;
-		m_queue.push_back(queued_packet());
-		queued_packet& qp = m_queue.back();
-		qp.ep = ep;
-		qp.flags = flags;
-		qp.buf.insert(qp.buf.begin(), p, p + len);
-		return true;
-	}
+	ptime now = time_now_hires();
+	time_duration delta = now - m_last_tick;
+	m_last_tick = now;
+
+	// add any new quota we've accrued since last time
+	m_quota += boost::uint64_t(m_rate_limit) * total_microseconds(delta) / 1000000;
+
+	// allow 3 seconds worth of burst
+	if (m_quota > 3 * m_rate_limit) m_quota = 3 * m_rate_limit;
+
+	// if there's no quota, and it's OK to drop, just
+	// drop the packet
+	if (m_quota < len && (flags & dont_drop) == 0) return false;
 
 	m_quota -= len;
+	if (m_quota < 0) m_quota = 0;
 	udp_socket::send(ep, p, len, ec, flags);
 	return true;
 }
 
-void rate_limited_udp_socket::on_tick(error_code const& e)
-{
-#if defined TORRENT_ASIO_DEBUGGING
-	complete_async("rate_limited_udp_socket::on_tick");
-#endif
-	if (e) return;
-	if (is_closed()) return;
-	error_code ec;
-	ptime now = time_now_hires();
-#if defined TORRENT_ASIO_DEBUGGING
-	add_outstanding_async("rate_limited_udp_socket::on_tick");
-#endif
-	m_timer.expires_at(now + seconds(1), ec);
-	m_timer.async_wait(boost::bind(&rate_limited_udp_socket::on_tick, this, _1));
-
-	time_duration delta = now - m_last_tick;
-	m_last_tick = now;
-	if (m_quota < m_rate_limit) m_quota += boost::uint64_t(m_rate_limit) * total_milliseconds(delta) / 1000;
-
-	if (m_queue.empty()) return;
-
-	while (!m_queue.empty() && int(m_queue.front().buf.size()) <= m_quota)
-	{
-		queued_packet const& p = m_queue.front();
-		TORRENT_ASSERT(m_quota >= int(p.buf.size()));
-		m_quota -= p.buf.size();
-		error_code ec;
-		udp_socket::send(p.ep, &p.buf[0], p.buf.size(), ec, p.flags);
-		m_queue.pop_front();
-	}
-}
-
-void rate_limited_udp_socket::close()
-{
-	error_code ec;
-	m_timer.cancel(ec);
-	udp_socket::close();
-}
-
