@@ -377,6 +377,9 @@ struct utp_socket_impl
 	// the address of the remote endpoint
 	address m_remote_address;
 
+	// the local address
+	address m_local_address;
+
 	// the send and receive buffers
 	// maps packet sequence numbers
 	packet_buffer m_inbuf;
@@ -739,7 +742,7 @@ utp_stream::endpoint_type utp_stream::local_endpoint(error_code& ec) const
 		ec = asio::error::not_connected;
 		return endpoint_type();
 	}
-	return m_impl->m_sm->local_endpoint(ec);
+	return tcp::endpoint(m_impl->m_local_address, m_impl->m_sm->local_port(ec));
 }
 
 utp_stream::~utp_stream()
@@ -1013,6 +1016,9 @@ void utp_stream::do_connect(tcp::endpoint const& ep, utp_stream::connect_handler
 	m_impl->m_port = ep.port();
 	m_impl->m_connect_handler = handler;
 
+	error_code ec;
+	m_impl->m_local_address = m_impl->m_sm->local_endpoint(m_impl->m_remote_address, ec).address();
+
 	if (m_impl->test_socket_state()) return;
 	m_impl->send_syn();
 }
@@ -2458,6 +2464,9 @@ bool utp_socket_impl::incoming_packet(char const* buf, int size
 				m_remote_address = ep.address();
 				m_port = ep.port();
 
+				error_code ec;
+				m_local_address = m_sm->local_endpoint(m_remote_address, ec).address();
+
 				m_ack_nr = ph->seq_nr;
 				m_seq_nr = random();
 				m_acked_seq_nr = (m_seq_nr - 1) & ACK_MASK;
@@ -2788,7 +2797,9 @@ void utp_socket_impl::do_ledbat(int acked_bytes, int delay, int in_flight, ptime
 
 	if (m_slow_start)
 	{
-		scaled_gain = (std::max)((window_factor * m_cwnd) >> 16, linear_gain);
+		// mimic TCP slow-start by adding the number of acked
+		// bytes to cwnd
+		scaled_gain = (std::max)(boost::int64_t(acked_bytes) << 16, linear_gain);
 	}
 	else
 	{
