@@ -199,7 +199,12 @@ void start_proxy(int port, int proxy_type)
 		, port, type, auth);
 
 	fprintf(stderr, "starting delegated proxy on port %d (%s %s)...\n", port, type, auth);
-	system(buf);
+	int r = system(buf);
+	if (r != 0)
+	{
+		fprintf(stderr, "failed (%d) %s\n", errno, strerror(errno));
+		exit(1);
+	}
 	fprintf(stderr, "launched\n");
 	// apparently delegate takes a while to open its listen port
 	test_sleep(500);
@@ -213,6 +218,35 @@ boost::intrusive_ptr<T> clone_ptr(boost::intrusive_ptr<T> const& ptr)
 	return boost::intrusive_ptr<T>(new T(*ptr));
 }
 
+void create_random_files(std::string const& path, const int file_sizes[], int num_files)
+{
+	error_code ec;
+	char* random_data = (char*)malloc(300000);
+	for (int i = 0; i != num_files; ++i)
+	{
+		std::generate(random_data, random_data + 300000, &std::rand);
+		char filename[200];
+		snprintf(filename, sizeof(filename), "test%d", i);
+		std::string full_path = combine_path(path, filename);
+		int to_write = file_sizes[i];
+		file f(full_path, file::write_only, ec);
+		if (ec) fprintf(stderr, "failed to create file \"%s\": (%d) %s\n"
+			, full_path.c_str(), ec.value(), ec.message().c_str());
+		size_type offset = 0;
+		while (to_write > 0)
+		{
+			int s = (std::min)(to_write, 300000);
+			file::iovec_t b = { random_data, s};
+			f.writev(offset, &b, 1, ec);
+			if (ec) fprintf(stderr, "failed to write file \"%s\": (%d) %s\n"
+				, full_path.c_str(), ec.value(), ec.message().c_str());
+			offset += s;
+			to_write -= s;
+		}
+	}
+	free(random_data);
+}
+
 boost::intrusive_ptr<torrent_info> create_torrent(std::ostream* file, int piece_size
 	, int num_pieces, bool add_tracker, bool encrypted_torrent)
 {
