@@ -226,27 +226,8 @@ int run_suite(char const* protocol, bool test_url_seed, bool chunked_encoding, b
 
 	if (test_url_seed)
 	{
-		char* random_data = (char*)malloc(300000);
-		for (int i = 0; i != sizeof(file_sizes)/sizeof(file_sizes[0]); ++i)
-		{
-			std::generate(random_data, random_data + 300000, &std::rand);
-			char filename[200];
-			snprintf(filename, sizeof(filename), "tmp1_web_seed/test_torrent_dir/test%d", i);
-			int to_write = file_sizes[i];
-			file f(filename, file::write_only, ec);
-			size_type offset = 0;
-			while (to_write > 0)
-			{
-				int s = (std::min)(to_write, 300000);
-				file::iovec_t b = { random_data, s};
-				f.writev(offset, &b, 1, ec);
-				offset += s;
-				to_write -= s;
-			}
-		}
-
+		create_random_files("tmp1_web_seed/test_torrent_dir", file_sizes, sizeof(file_sizes)/sizeof(file_sizes[0]));
 		add_files(fs, "tmp1_web_seed/test_torrent_dir");
-		free(random_data);
 	}
 	else
 	{
@@ -302,25 +283,7 @@ int run_suite(char const* protocol, bool test_url_seed, bool chunked_encoding, b
 		// corrupt the files now, so that the web seed will be banned
 		if (test_url_seed)
 		{
-			char* random_data = (char*)malloc(300000);
-			for (int i = 0; i != sizeof(file_sizes)/sizeof(file_sizes[0]); ++i)
-			{
-				std::generate(random_data, random_data + 300000, &std::rand);
-				char filename[200];
-				snprintf(filename, sizeof(filename), "tmp1_web_seed/test_torrent_dir/test%d", i);
-				int to_write = file_sizes[i];
-				file f(filename, file::write_only, ec);
-				size_type offset = 0;
-				while (to_write > 0)
-				{
-					int s = (std::min)(to_write, 300000);
-					file::iovec_t b = { random_data, s};
-					f.writev(offset, &b, 1, ec);
-					offset += s;
-					to_write -= s;
-				}
-			}
-			free(random_data);
+			create_random_files("tmp1_web_seed/test_torrent_dir", file_sizes, sizeof(file_sizes)/sizeof(file_sizes[0]));
 		}
 		else
 		{
